@@ -1,6 +1,6 @@
 "All components of the public interface"
 
 from .control import Control
-from .models import Network
+from .models import Profile
 
-__all__ = ['Control', 'Network']
+__all__ = ['Control', 'Profile']
