@@ -28,7 +28,9 @@ class Control:
 
     def close(self) -> None:
         "Close all interfaces"
+        LOGGER.info('Closing interfaces')
         if self._interfaces is None:
+            LOGGER.debug('No interfaces')
             return
         for iface in self._interfaces:
             iface.close()
@@ -36,10 +38,12 @@ class Control:
 
     def interface(self, name: str, **kwargs) -> Interface:
         "Get specific interface"
+        LOGGER.info('Connecting to interface %s', name)
         return Interface(self, name, **kwargs)
 
     def interface_names(self):
         "List of interface names"
+        LOGGER.info('Returning names of %i interfaces', len(self.interfaces))
         return [
             interface.name for interface in self.interfaces
         ]
@@ -47,6 +51,7 @@ class Control:
     @property
     def interfaces(self) -> List[Interface]:
         "List of interfaces"
+        LOGGER.info('Listing interfaces')
         if self._interfaces is None:
             self._interfaces = []
             for name in find_sockets(self._sock_path):
