@@ -1,18 +1,20 @@
 "Communication with wpa_supplicant interface"
+# pylint: disable=too-many-instance-attributes
 
 import os
 import time
-import tempfile
 import threading
 import socket
 import logging
 
 from select import select
 from typing import List, Union
-from os.path import join as pathjoin
+from os.path import join as pathjoin, dirname
 
-from .utils import tempnam, is_sock, safe_encode
-from .models import InterfaceStatus, Network, deserialize_networks
+from .utils import tempnam, is_sock, safe_encode, SOCKET_PREFIX
+from .models import (
+    InterfaceStatus, Profile, deserialize_scanned,
+)
 
 
 LOGGER = logging.getLogger(__name__)
@@ -36,18 +38,18 @@ class _BackgroundScan:
         self._t = None
 
     def _scan(self, callback, timeout):
-        "Background scan thread entry point"
+        "Background scan thread entry point."
         networks, started = set(), time.time()
         self._interface.scan()
         while self._running and time.time() - started < timeout:
             time.sleep(1.0)
-            for network in self._interface.results():
+            for network in self._interface.scan_results():
                 if network.ssid not in networks:
                     networks.add(network.ssid)
                     callback(network)
 
     def start(self, callback: callable, timeout: float=SCAN_TIMEOUT):
-        "Start background scan"
+        "Start background scan."
         assert callable(callback), 'Callback must be callable'
         self._running = True
         self._t = threading.Thread(
@@ -55,7 +57,7 @@ class _BackgroundScan:
         self._t.start()
 
     def stop(self):
-        "Stop background scan"
+        "Stop background scan."
         if self._t is None:
             return
         self._running = False
@@ -107,20 +109,32 @@ class Interface:
         self._server_path = pathjoin(self._control._sock_path, self.name)
         assert is_sock(self._server_path), 'Not a valid interface'
         self._client_path = None
+        self._scanned = {}
+        self._profiles = {}
 
     def __del__(self):
         self.close()
 
     @property
     def name(self):
-        "This interface's name"
+        "This interface's name."
         return self._name
 
     @property
     def control(self) -> 'Control':
-        "The parent object which gives access to additional interfaces"
+        "The parent object which gives access to additional interfaces."
         return self._control
 
+    @property
+    def profiles(self):
+        "Networks in wpa_supplicant.conf"
+        return list(self._profiles.values())
+
+    @property
+    def scanned(self):
+        "Networks found via scan()."
+        return list(self._scanned.values())
+
     def close(self) -> None:
         """
         Close the socket when deallocated.
@@ -142,7 +156,7 @@ class Interface:
         """
         if self._connection is not None:
             return
-        self._client_path = tempnam(tempfile.tempdir, 'pywpas')
+        self._client_path = tempnam(dirname(self._server_path), SOCKET_PREFIX)
         self._connection = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
         self._connection.bind(self._client_path)
         self._connection.connect(self._server_path)
@@ -183,61 +197,98 @@ class Interface:
         return resp.split(b'\n')
 
     def ping(self) -> None:
-        "Connection test"
+        "Connection test."
+        LOGGER.info('Pinging wpa_supplicant')
         resp = self._send_and_recv(b'PING')
         assert resp == [b'PONG'], 'Did not receive proper reply'
 
     def status(self) -> InterfaceStatus:
-        "Get interface status"
+        "Get interface status."
+        LOGGER.info('Retrieving interface status')
         status = InterfaceStatus.deserialize(self._send_and_recv('STATUS'))
-        LOGGER.info('Interface status: %s', status)
         return status
 
     def scan(self) -> None:
-        "Start scanning"
-        LOGGER.info('Scanning')
+        "Start scanning."
+        LOGGER.info('Initiating scan')
         self._send(b'SCAN')
 
     def background_scan(self, callback: callable,
                         timeout: float=SCAN_TIMEOUT) -> None:
-        "Perform background scan on thread"
+        "Perform background scan on thread."
+        LOGGER.info('Starting background scan')
         scan = _BackgroundScan(self)
         scan.start(callback, timeout)
         return scan
 
-    def results(self):
-        "Return scan results"
-        networks = deserialize_networks(self._send_and_recv(b'SCAN_RESULTS'))
+    def scan_results(self):
+        "Return scan results."
+        LOGGER.info('Retrieving scan results')
+        networks = deserialize_scanned(self._send_and_recv(b'SCAN_RESULTS'))
         for network in networks:
             LOGGER.info('Found network: %s', network)
-        return networks
+            self._scanned[network.ssid] = network
+        return self.scanned
 
-    def add_network(self, network: Network) -> None:
-        "Add network profile"
+    def remove_results(self):
+        "Remove scan results."
+        self._scanned.clear()
 
-    def networks(self) -> List[Network]:
-        "List network profiles"
+    def add_network(self, profile: Profile) -> None:
+        "Add network profile."
+        LOGGER.info('Adding network: %s', profile.ssid)
+        profile.id = int(self._send_and_recv(b'ADD_NETWORK')[0])
+        LOGGER.debug('Assigned id: %i', profile.id)
+        self._send(f'SET_NETWORK {profile.id} ssid {profile.ssid}')
+        self._send(f'SET_NETWORK {profile.id} key_mgmt {profile.key_mgmt}')
+        self._send(f'SET_NETWORK {profile.id} proto {profile.proto}')
+        self._send(f'SET_NETWORK {profile.id} psk {profile.psk}')
+        self._profiles[profile.id] = profile
 
-    def remove_network(self, network: Network) -> None:
-        "Remove given network profile"
-        self._send(b'REMOVE_NETWORK %s' % safe_encode(network.ssid))
+    def list_networks(self) -> List[Profile]:
+        "List network profiles."
+        LOGGER.info('Listing networks')
+        profile_ids = map(int, self._send_and_recv(b'LIST_NETWORKS'))
+        LOGGER.debug('Received network ids: %s', profile_ids)
+        for profile_id in profile_ids:
+            kwargs = {}
+            kwargs['ssid'] = self._send_and_recv(f'GET_NETWORK {profile_id} ssid')[0]
+            kwargs['key_mgmt'] = self._send_and_recv(f'GET_NETWORK {profile_id} key_mgmt')[0]
+            kwargs['proto'] = self._send_and_recv(f'GET_NETWORK {profile_id} proto')[0]
+            kwargs['ciphers'] = self._send_and_recv(f'GET_NETWORK {profile_id} pairwise')[0]
+            self._profiles[profile_id] = Profile(**kwargs)
+        return self.profiles
+
+    def remove_network(self, profile: Profile) -> None:
+        "Remove given network profile."
+        LOGGER.info('Removing network profile: %s', profile.ssid)
+        self._send(f'REMOVE_NETWORK {profile.id}')
+        self._profiles.pop(profile.id, None)
 
     def remove_networks(self) -> None:
-        "Delete all network profiles"
-        LOGGER.info('Removing all networks')
+        "Delete all network profiles."
+        LOGGER.info('Removing all network profiles')
         self._send(b'REMOVE_NETWORK all')
+        self._profiles.clear()
 
-    def connect(self, network: Network):
-        "connect interface to given network"
+    def connect(self, profile: Profile):
+        "connect interface to given network."
+        if profile.id is None:
+            self.add_network(profile)
+        LOGGER.info('Connecting to network: %s', profile.ssid)
+        self._send_and_recv(f'SELECT_NETWORK {profile.id}')
 
     def disconnect(self) -> None:
-        "Disconnect interface"
+        "Disconnect interface."
+        LOGGER.info('Disconnecting')
         self._send(b'DISCONNECT')
 
     def save_config(self):
-        "Save running config to file"
-        self._send('SAVE_CONFIG')
+        "Save running config to file."
+        LOGGER.info('Saving configuration')
+        self._send(b'SAVE_CONFIG')
 
     def stop_ap(self):
-        "Stop access point"
-        self._send('STOP_AP')
+        "Stop access point."
+        LOGGER.info('Stopping access point')
+        self._send(b'STOP_AP')
