@@ -3,10 +3,17 @@
 import os
 import tempfile
 import stat
+import logging
 
 from os.path import join as pathjoin
 
 
+SOCKET_PREFIX = 'pywpas'
+
+LOGGER = logging.getLogger(__name__)
+LOGGER.addHandler(logging.NullHandler())
+
+
 def safe_decode(val):
     "Try to decode bytes to str"
     try:
@@ -39,9 +46,15 @@ def tempnam(dir: str, prefix: str='') -> str:  # pylint: disable=redefined-built
 
 def is_sock(path):
     "Checks if given path is a socket"
+    LOGGER.debug('Checking if %s is a socket', path)
     return stat.S_ISSOCK(os.stat(path).st_mode)
 
 
 def find_sockets(path):
     "Finds sockets in given directory"
-    return [n for n in os.listdir(path) if is_sock(pathjoin(path, n))]
+    LOGGER.info('Searching directory %s for sockets', path)
+    # Filter out non-sockets, and other potential client sockets.
+    return [
+        n for n in os.listdir(path) \
+            if is_sock(pathjoin(path, n)) and not n.startswith(SOCKET_PREFIX)
+    ]
