@@ -1,7 +1,7 @@
 from unittest import TestCase
 
 from pywpas.models import (
-    InterfaceStatus, Network, deserialize_networks,
+    InterfaceStatus, Scanned, Profile, deserialize_scanned,
 )
 
 
@@ -33,12 +33,18 @@ SCAN_RESULTS = b'bssid / frequency / signal level / flags / ssid\n' \
 
 
 class NetworkTestCase(TestCase):
-    def test_deserialize_networks(self):
-        networks = deserialize_networks(SCAN_RESULTS.split(b'\n'))
+    def test_deserialize_scanned(self):
+        networks = deserialize_scanned(SCAN_RESULTS.split(b'\n'))
         self.assertEqual(11, len(networks))
         self.assertEqual(
-            'bssid=08:02:8e:9c:9d:15, frequency=2452, signal_level=-36, flags=[WPA2-PSK-CCMP][ESS], ssid=NachoWIFI',
+            'bssid=08:02:8e:9c:9d:15, frequency=2452, signal_level=-36, flags=[WPA2-PSK-CCMP][ESS], '
+            'ssid=NachoWIFI',
             str(networks[0]))
+        profile = networks[0].as_profile(psk='Super secret!')
+        self.assertEqual(
+            'id=None, ssid=NachoWIFI, key_mgmt=None, proto=None, ciphers=None, '
+            'psk=Super secret!',
+            str(profile))
 
     def test_deserialize_interfacestatus(self):
         status = InterfaceStatus.deserialize(INTERFACE_STATUS.split(b'\n'))
